/*report bugs:damien.vdd@gmail.com
Version 0.23, changes:
	- Corrected a formatting problem in by() and twoway() sheets if a grouping variable was date-formatted
Version 0.22, changes:
	- Corrected bug appearing in "coverage" sheet: never missing variables where wrongly given null sample sizes at the aggregate level
Version 0.21, changes:
	- Specified MI and ME as option abbreviations for Missings and Means, respectively
	- Allowed "by" variables to have missing values
	- Added "(No Grouping)" value for the column "GroupingVariables" in the coverage sheet, when no option "by" was specified.
Version 0.20, changes:
	- Removed column "MissingSD" of coverage sheet, replaced with "SampleSize"; changed names of other columns
Version 0.19, changes:
	- Corrected a bug displaying several "Variable" columns in the twoway sheets, dating back from v0.17
Version 0.18, changes:
	- Corrected a bug preventing the display of variable labels, dating from version 0.16...
Version 0.17, changes:
	- Prevented a bug in the coverage sheet when "missings" and "by" options were both specified
	- Corrected a bug when the dataset active in memory included a variable called "variable"
Version 0.16, changes:
	- Corrected a bug when variable labels had parentheses or "`" within them
Version 0.15, changes:
	- Added option Means, which calculates the means of varlist, instead of coverage binary or missing ratios. Useful to display many stats.
	- Allowed the over variable to have missing values
Version 0.14, changes:
	- Added option over(varname), which allows to disaggregate the output excel file by the values taken by varname
Version 0.13, changes:
	- Corrected a stupid bug that prevented the program to work if no by() option where specified
version 0.12, changes:
	- Added option [Missings], which gives the missing ratios per aggregate levels, instead of the coverage binaries, when "missings" is specified
version 0.11, changes:
	- Corrected bug where the same variable appeared several times at aggregate levels
	- Corrected bug when an allways missing variable was specified in varlist

This program generates an excel file with informations on the coverage of variables within a survey

Features to correct/implement
	- Find a way to  withdraw "misstable patterns" to make faster computations
	- Find a way to find time to rewrite everything for parsimony, it's too long to compute godamnit!
	- Prevent a bug when variables have too many values for Excel in the twoway sheet (warning, limits depend on the version of Excel)
	- Prevent a bug when a sheetname is too long  (warning, limits depend on the version of Excel)(hint: use abbrev())
	- Prevent a bug when a sheet's rows are too great (warning, limits depend on the version of Excel)
	- Investigate what happens if by() variables are individual-level
	- Compute figures at the aggregate level of over()
	- Consider dropping "misstable summarize varlist,gen()" and replace with recode varlist (missing=1)(nonmissing=0), or with count if
	- Recode everything to authorize string variables in varlist? -_-
*/

program define coverage
version 13	//untested with prior versions of Stata
syntax varlist(min=1 numeric) [,BYgroup(varlist fv numeric min=1) Decimals(integer 3) MIssings MEans Over(varname numeric min=1 max=1)]
tempfile main twoway twoway_agg overaggby overaggtwoway labeloverdo
qui save `main',replace all orphans
capture noisily{ //begin capture noisily
if "`missings'"!="" & "`means'"!=""{
	di as err `"Options "Missings" and "Means" cannot be combined"'
	exit 184
}
if `decimals'<=0 local decimals=0
local round=1/(10^`decimals')
local c_date: display c(current_date)
local c_time: display c(current_time)
local c_time_date = "`c_date'"+"_" +"`c_time'"
local now = subinstr("`c_time_date'", ":", "", .)
local now = subinstr("`now'", " ", "", .)
local unabvarlist `varlist'
_fv_check_depvar `unabvarlist'
local unabvarlist: list uniq local(unabvarlist)
if "`bygroup'"==""{ //beginning of bygroup definition
	local ngroups=0
	local group0 (No grouping)
	if "`missings'"!="" di `"Note: option "Missings" is useless without a by() option"'
}
else{ 
	fvrevar `bygroup',list
	local groupvars `r(varlist)'
	qui ds `groupvars',has(format %t*)
	local bydates `r(varlist)'
	if "`bydates'"!=""{
		local nbydates: list sizeof local(bydates)
		if `nbydates'>1 di as err "Please standardize or suppress the date formats of the by() variables if they don't appear properly in Excel"
		local firstbydate: word 1 of `bydates'
		local formatbydate: format `firstbydate'
		local datestring datestring("`formatbydate'")
	}
	local bygroup: list uniq bygroup
	local ngroups: word count `bygroup'
	local group0 (No grouping)
	forvalues i=1/`ngroups'{
		local group: word `i' of `bygroup'
		fvrevar `group',list
		local group`i' `r(varlist)'
		local nwaygroup`i': word count `group`i''
	}
} //end of bygroup definition
local forbiddenvarlist: list local(groupvars) | local(over)
local intersection: list local(unabvarlist) & local(forbiddenvarlist)
if "`intersection'"!=""{
	di `"Note: Grouping/Over variables "`intersection'" have been withdrawn from varlist automatically"'
	local unabvarlist: list local(unabvarlist) - local(intersection)
}
local row=1
if "`over'"=="" local nboverlevels=1 //beginning of over definition
else{
	local labelover: value label `over'
	qui label save `labelover' using `labeloverdo'
	qui levelsof `over',local(overlevels) missing
	local nboverlevels: word count `overlevels'
	if `nboverlevels'==1{
		di `"Note: `nboverlevels' has only one nonmissing value: "over" option is cancelled"'
		local over
	}
} //end of over definition
tempvar variables
forvalues o=1/`nboverlevels'{
	if "`over'"!=""{
		local overlevel`o': word `o' of `overlevels'
		local if`o' if `over'==`overlevel`o''
		if `o'==1 qui keep `if`o''
		else qui use `if`o'' using `main',clear
		local overlabel`o': label (`over') `overlevel`o'', strict
	}
	forvalues k=0/`ngroups'{ //begin looping among aggregate levels
		if `k'!=0{
			qui use `if`o'' using `main' ,clear
			if "`missings'"!="" qui recode `unabvarlist' (nonmissing=0)(missing=1)
			collapse (mean) `unabvarlist', by(`group`k'') fast
				preserve
				if "`missings'"=="" & "`means'"=="" qui recode `unabvarlist' (nonmissing=1)(missing=0)
				if "`over'"!=""{
					tempfile overby`o'`k'
					qui save `overby`o'`k'',replace
				}
				else qui export excel * using coverage_`now'.xls, sheet("by(`group`k'')") sheetreplace firstrow(variables) `datestring'
				if "`nwaygroup`k''"=="2"{
					qui save `twoway',replace
					local vertvar: word 1 of `group`k''
					local horivar: word 2 of `group`k''
					local notfirst=0
					foreach valuvar of varlist `unabvarlist'{ //begin looping among variables for by() and twoway() sheets
						if "`notfirst'"=="1" qui use `twoway',clear
						keep `vertvar' `horivar' `valuvar'
						tempvar horivarint
						egen int `horivarint'= group(`horivar'), label lname(`horivar'labels) missing
						drop `horivar'
						qui levelsof `horivarint',local(numlist)
						foreach id of local numlist{
							local label`id': label `horivar'labels `id'
						}
						qui reshape wide `valuvar',i(`vertvar')j(`horivarint')
						foreach id of local numlist{
							la var `valuvar'`id' "`label`id''"
							local label`id'=strtoname("`label`id''")
							ren `valuvar'`id' `horivar'_`label`id''
						}
						gen str `variables'="`valuvar'"
						
						if "`notfirst'"=="1" qui append using `twoway_agg'
						qui save `twoway_agg',replace
						local notfirst=1
					} //end looping among variables for by() and twoway() sheets
					if "`over'"!=""{
						tempfile overtwoway`o'`k'
						local overvarvert`k' `over' `variables' `vertvar'
						qui save `overtwoway`o'`k'',replace
					}
					else{
						la var `variables' "Variables"
						order `variables' `vertvar',first
						sort `variables' `vertvar'
						qui export excel * using coverage_`now'.xls, sheet("twoway(`group`k'')") firstrow(varlabels) `datestring'
					}
				}
				restore
		}
		tempvar mv`k'
		qui misstable summarize `unabvarlist',generate(`mv`k'') all
		qui misstable pattern `unabvarlist'
		local missingvarlist `r(vars)'
		local nonmissingvarlist: list local(unabvarlist) - local(missingvarlist)
		local varlist`k' `nonmissingvarlist'
		local nmissingvars: word count `missingvarlist'
		forvalues i=1/`nmissingvars'{
			local varname: word `i' of `missingvarlist'
			if `i'==1 qui replace `varname'=`mv`k''`varname'
			else qui replace `varname'=((`mv`k''`varname'+`prior')>0)
			local prior `varname'
			local varlist`k' `varlist`k'' `varname'
		}
		foreach var of local varlist`k'{ //begin looping among variables for coverage sheet
			if `k'==0 {
				local varlab`var': variable label `var'
				local varlab`var'=subinstr("`varlab`var''","(","[",.)
				local varlab`var'=subinstr("`varlab`var''",")","]",.)
				local varlab`var'=subinstr("`varlab`var''","`","'",.)
			}
			local nonmissingvar: list local(var) in local(nonmissingvarlist)
			local row=`row'+1
			if "`missings'"!="" & `k'>0 {
				qui putexcel A`row'=("`var'") B`row'=("`varlab`var''") C`row'=("NA with 'missings' option") G`row'=("`group`k''") H`row'=(_N) using coverage_`now'.xls, sheet(coverage) modify
				if "`over'"!="" qui putexcel I`row'=("`overlevel`o'' `overlabel`o''") using coverage_`now'.xls, sheet(coverage) modify
				continue	//this stops foreach=var and goes to the next var
			}
			if `nonmissingvar'==1 {
				qui putexcel A`row'=("`var'") B`row'=("`varlab`var''") C`row'=(0) D`row'=(_N) E`row'=(0) F`row'=(_N) G`row'=("`group`k''") H`row'=(_N) using coverage_`now'.xls, sheet(coverage) modify
				if "`over'"!="" qui putexcel I`row'=("`overlevel`o'' `overlabel`o''") using coverage_`now'.xls, sheet(coverage) modify
				continue	//this stops foreach=var and goes to the next var
			}
			qui su `mv`k''`var'
			qui putexcel C`row'=(round(`r(mean)',`round')) D`row'=((1-`r(mean)')*_N) using coverage_`now'.xls, sheet(coverage) modify
			if "`over'"!="" qui putexcel I`row'=("`overlevel`o'' `overlabel`o''") using coverage_`now'.xls, sheet(coverage) modify
			qui su `var'
			if `r(N)'>0{
				local mean=`r(mean)'
				local target=`r(N)'*(1-`r(mean)')
			}
			else{
				local mean=1
				local target=0
			}
			qui putexcel A`row'=("`var'") B`row'=("`varlab`var''") E`row'=(round(`mean',`round')) F`row'=(round(`target')) G`row'=("`group`k''") H`row'=(_N) using coverage_`now'.xls, sheet(coverage) modify
		} //end looping among variables for coverage sheet
	} //end looping among by groups
} //end of looping among over levels
qui putexcel A1=("Variable") B1=("VariableLabel") C1=("MissingRatio") D1=("SampleSize") E1=("MissingRatioCumulated") F1=("SampleSizeCumulated") G1=("GroupingVariables") H1=("TotalSampleSize") using coverage_`now'.xls, sheet(coverage) modify
if "`over'"!=""{
	qui putexcel I1=("Over(`over')") using coverage_`now'.xls, sheet(coverage) modify
	forvalues k=1/`ngroups'{
		forvalues o=1/`nboverlevels'{
			qui use `overby`o'`k'',clear
			qui gen `over'=`overlevel`o''
			if `o'>1 qui append using `overaggby'
			qui save `overaggby',replace
		}
		qui run `labeloverdo'
		label values `over' `labelover'
		order `over',first
		sort *
		qui export excel * using coverage_`now'.xls, sheet("by(`group`k'')") sheetreplace firstrow(variables) `datestring'
		if "`nwaygroup`k''"=="2"{
			forvalues o=1/`nboverlevels'{
				qui use `overtwoway`o'`k'',clear
				qui gen `over'=`overlevel`o''
				if `o'>1 qui append using `overaggtwoway'
				qui save `overaggtwoway',replace
			}
			qui run `labeloverdo'
			label values `over' `labelover'
			order *,seq
			order `overvarvert`k'',first
			sort `overvarvert`k''
			la var `variables' "Variables"
			qui export excel * using coverage_`now'.xls, sheet("twoway(`group`k'')") firstrow(varlabels) `datestring'
		}
	}
}

} //end of capture noisily
qui use `main',clear
if _rc==0 di `"The output excel file is named "coverage_`now'.xls", located in "`c(pwd)'""'
else{
	local rc=_rc
	cap rm coverage_`now'.xls
	di as err "The program ended with an error"
	exit `rc'
}
end

